#include <conio.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <afx.h>
#include <io.h>
#include <direct.h>

typedef unsigned char  byte;
typedef unsigned short word;

struct bitmap_header
{
    unsigned short type;
    unsigned short lof;
    unsigned short lof2;
    unsigned short x_hot;
    unsigned short y_hot;
    unsigned short first_pixel;
    unsigned short first_pixel2;
    unsigned short header_size;
    unsigned short header_size2;
    unsigned short x_size;             //10
    unsigned short x_size2;
    unsigned short y_size;
    unsigned short y_size2;
    unsigned short target;
    unsigned short bits_per_pixel;      //10
    
    unsigned short compression_method;
    unsigned short compression_method2;
    unsigned short compressed_size;
    unsigned short compressed_size2;
    unsigned short x_res;              //10
    unsigned short x_res2;
    unsigned short y_res;
    unsigned short y_res2;
    unsigned short used_clrs;
    unsigned short used_clrs2;         //10
    unsigned short important_clrs;
    unsigned short important_clrs2;
    char * palette;
    char * pixels;
}bmfh;

struct colour
{
    unsigned char r,g,b;
};

struct palette
{
    struct colour col[256];
}pal;

byte * px;
int ix,iy;

void usage()
{
	printf("\n  USAGE:");
	printf("\n");
	printf("\n 1. Create a new folder.");
	printf("\n 1. Copy several texture files (.map) to the folder. ");
	printf("\n 2. Copy one palette file (.act) to the same folder. ");
	printf("\n 3. Drag and drop the folder onto this exe.");
	printf("\n");
	printf("\n For each .map file in the folder, a .bmp file will be created.");
	printf("\n If multiple .act files exist, the program just uses the first one it sees");
	printf("\n");
	printf("\n Any key to exit...\n");
	getch();
	exit(0);
}



void make_bmfh(int w, int h)
{
    int b;//buffer var
    //write BMP header...
    bmfh.type=(77<<8)+66;
    b=1078+(w*h);
    bmfh.lof=b&65535;
    bmfh.lof2=b>>16;
    bmfh.x_hot=0;
    bmfh.y_hot=0;
    bmfh.first_pixel=1078;
    bmfh.first_pixel2=0;
    bmfh.header_size=40;
    bmfh.header_size2=0;
    bmfh.x_size=w;
    bmfh.x_size2=0;
    bmfh.y_size=h;
    bmfh.y_size2=0;
    bmfh.target=1;
    bmfh.bits_per_pixel=8;
    
    bmfh.compression_method=0;
    bmfh.compression_method2=0;
    bmfh.compressed_size=0;
    bmfh.compressed_size2=0;
    bmfh.x_res=666;
    bmfh.x_res2=0;
    bmfh.y_res=666;
    bmfh.y_res2=0;
    bmfh.used_clrs=0;
    bmfh.used_clrs2=0;
    bmfh.important_clrs=0;
    bmfh.important_clrs2=0;
    
}




void load_map(char *fname)
{
    FILE *input;
    input=fopen(fname,"r+b");
	fread(&ix,1,4,input);
	fread(&iy,1,4,input);
	px=(byte *)malloc(ix*iy);
	fread(px,ix*iy,1,input);
	printf ("Loaded %s, size %i x %i...", fname, ix, iy);
    fclose(input);
}



void save_bmp(char *fname)
{
    int i;
    FILE *output;
    output=fopen(fname,"w+b");
    fwrite(&bmfh,2,27,output);
    if (bmfh.bits_per_pixel==8)
    {
        for(i=0;i<256;i++)
        {
          putc(pal.col[i].b,output);
          putc(pal.col[i].g,output);
          putc(pal.col[i].r,output);
          putc(0,output);
        }
        fwrite(px,bmfh.x_size*bmfh.y_size,1,output);
    }
    fclose(output);
	printf ("  Saved %s.\n", fname);

}

void do_file(char *fname)
{
	char *output;

	load_map(fname);
	make_bmfh(ix,iy);
	output = strlwr(strdup(fname));
	strcpy(strstr(output,".map"),".bmp");
	save_bmp(output);
	free(px);
}


void load_palette()
{
	int i;
	long hFile;
	static     _finddata_t fileinfo;
	FILE *input;

	if( (hFile = _findfirst( "*.act", &fileinfo)) == -1L )
	{
		printf("\nError: No palette found. Please copy palettes to the folder and try again.\n");
		usage();
	}
	else
	{
		printf("Loading palette %s\n\n", fileinfo.name );
		input=fopen(fileinfo.name,"rb");
		for(i=0;i<256;i++)
		{
			pal.col[i].r=getc(input);
			pal.col[i].g=getc(input);
			pal.col[i].b=getc(input);
		} 
		fclose(input);
	}
}


void do_dir()
{
	int i=1;
	long hFile;
	static     _finddata_t fileinfo;
	if( (hFile = _findfirst( "*.map", &fileinfo)) == -1L )
	{
		printf("\nError: No maps found. Please copy map to the folder and try again.\n");
		usage();
	}

	do_file(fileinfo.name);
	while( _findnext( hFile, &fileinfo ) == 0 )
	{
		do_file(fileinfo.name);
		i++;
	}
	printf("\n\n %i files processed OK.\n",i);
}




void main(int argc, char*argv[])
{
	int i=1;
    printf("  Map 2 Bmp - Batch Texture Conversion Utility for BZ editors.\n\n");

    if (argc==1)
	{
		printf("\nError: No folder specified.\n");
		usage();
	}
	for (i=1; i<argc; i++)
	{
		if( _chdir( argv[i] ))
		{
			printf( "%s is not a folder. Skipping.\n", argv[i] );
		}
		else
		{
			printf("\nProcessing %s.\n",argv[i]);
			load_palette();
			do_dir();
		}
	}
	printf("\n Done. ");
	printf("\n Any key to exit...\n");
	getch();
	exit(0);
}

